/*
 *	$Id: scan_keyb.c,v 1.1.2.1 2003/02/02 16:20:59 idkfa Exp $ 
 *	Copyright (C) 2000 YAEGASHI Takeshi
 *	Copyright (C) 2003 Andriy Skulysh
 *	Generic scan keyboard driver
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/kbd_kern.h>
#include <linux/input.h>
#include <linux/timer.h>

#define SCANHZ	(HZ/20)

struct scan_keyboard {
	struct scan_keyboard *next;
	int (*scan)(unsigned char *buffer);
	const unsigned char *table;
	unsigned char *s0, *s1;
	int length;
	struct input_dev dev;
};

static int scan_jiffies=0;
static struct scan_keyboard *keyboards=NULL;
struct timer_list scan_timer;
static char *hpkbd_name = "Hitachi scankeyboard";
static char *hpkbd_phys = "input0";

static void check_kbd(struct scan_keyboard *kbd,
		      unsigned char *new, unsigned char *old)
{
	unsigned char *table=kbd->table;
	int length=kbd->length;
	int need_tasklet_schedule=0;
	unsigned int xor, bit;
	
	while(length-->0) {
		if((xor=*new^*old)==0) {
			table+=8;
		}
		else {
			for(bit=0x01; bit<0x100; bit<<=1) {
				if(xor&bit) {
					input_report_key(&kbd->dev,*table,!(*new&bit));
					need_tasklet_schedule=1;
#if 0
					printk("0x%x %s\n", *table, (*new&bit)?"released":"pressed");
#endif
				}
				table++;
			}
		}
		new++; old++;
	}

	if(need_tasklet_schedule)
	{
		input_sync(&kbd->dev);
		tasklet_schedule(&keyboard_tasklet);
	}
}


static void scan_kbd(unsigned long dummy)
{
	struct scan_keyboard *kbd;

	scan_jiffies++;

	for(kbd=keyboards; kbd!=NULL; kbd=kbd->next) {
		if(scan_jiffies&1) {
			if(!kbd->scan(kbd->s0))
				check_kbd(kbd, kbd->s0, kbd->s1);
			else
				memcpy(kbd->s0, kbd->s1, kbd->length);
		}
		else {
			if(!kbd->scan(kbd->s1))
				check_kbd(kbd, kbd->s1, kbd->s0);
			else
				memcpy(kbd->s1, kbd->s0, kbd->length);
		}
		
	}

	init_timer(&scan_timer);
	scan_timer.expires = jiffies + SCANHZ;
	scan_timer.data = 0;
	scan_timer.function = scan_kbd;
	add_timer(&scan_timer);
}


int register_scan_keyboard(int (*scan)(unsigned char *buffer),
			   const unsigned char *table,
			   int length)
{
	struct scan_keyboard *kbd;
	int i;

	kbd = kmalloc(sizeof(struct scan_keyboard), GFP_KERNEL);
	if (kbd == NULL)
		goto error_out;

	kbd->scan=scan;
	kbd->table=table;
	kbd->length=length;

	kbd->s0 = kmalloc(length, GFP_KERNEL);
	if (kbd->s0 == NULL)
		goto error_free_kbd;

	kbd->s1 = kmalloc(length, GFP_KERNEL);
	if (kbd->s1 == NULL)
		goto error_free_s0;

	memset(kbd->s0, -1, kbd->length);
	memset(kbd->s1, -1, kbd->length);
	
	memset(&(kbd->dev),0,sizeof(struct input_dev));
	kbd->dev.name=hpkbd_name;
	kbd->dev.phys=hpkbd_phys;
	kbd->dev.evbit[0] = BIT(EV_KEY) | BIT(EV_REP);
	init_input_dev(&kbd->dev);
	kbd->dev.keycode = table;
	kbd->dev.keycodesize = sizeof(unsigned char);
	kbd->dev.keycodemax = ARRAY_SIZE(table);
	for(i=0; i<128; i++)
		if(table[i]) set_bit(table[i], kbd->dev.keybit);
	clear_bit(0, kbd->dev.keybit);
	input_register_device(&kbd->dev);
	
	kbd->next=keyboards;
	keyboards=kbd;

	return 0;

 error_free_s0:
	kfree(kbd->s0);

 error_free_kbd:
	kfree(kbd);

 error_out:
	return -ENOMEM;
}
			      
			      
void __init scan_kbd_init(void)
{
	init_timer(&scan_timer);
	scan_timer.expires = jiffies + SCANHZ;
	scan_timer.data = 0;
	scan_timer.function = scan_kbd;
	add_timer(&scan_timer);

	printk(KERN_INFO "Generic scan keyboard driver initialized\n");
}
